freeslot("MT_HEATBEAM", "SPR_CHR2")

local CHARGE_TIME = TICRATE -- 1 second to charge
local FIRE_INTERVAL = 1 -- Fire every single tic
local HEAT_BEAM_SPEED = 60 -- Speed of the heat beam
local RING_DRAIN_DELAY = 1 * TICRATE -- Start draining rings after 1 second
local RING_DRAIN_INTERVAL = TICRATE / 10 -- Drain rings every 0.1 seconds
local MAX_FIRE_TIME = 1 * TICRATE -- Maximum 1 second of firing (updated to 1 second)

local playerChargeState = {} -- Table to track each player's charge state

local function HeatBeam(player)
    if not server then return end

    for player in players.iterate do
        if not (player.mo and player.mo.valid) then continue end
        if player.mo.skin ~= "nesgodzilla" then continue end

        -- Prevent using heat beam while in S_GODZILLA_ROAR state
        if player.mo.state == S_GODZILLA_ROAR then
            continue
        end

        if not playerChargeState[player] then
            playerChargeState[player] = {
                chargeTicks = 0,
                fireTicks = 0,
                isFiring = false,
                soundPlayed = false,
                ringDrainTicks = 0,
                fireDuration = 0 -- Track firing time
            }
        end

        local state = playerChargeState[player]

        if player.rings <= 1 then
            if state.isFiring then
                state.chargeTicks = 0
                state.isFiring = false
                state.fireTicks = 0
                state.soundPlayed = false
                state.ringDrainTicks = 0
                state.fireDuration = 0
                player.mo.sprite = SPR_PLAY
                player.mo.frame = A
            end
            continue
        end

        if player.cmd.buttons & BT_ATTACK ~= 0 then
            state.chargeTicks = state.chargeTicks + 1

            if state.chargeTicks < CHARGE_TIME then
                player.mo.sprite = SPR_CHR2
                player.mo.frame = A
            else
                player.mo.sprite = SPR_CHR2
                player.mo.frame = B

                if not state.isFiring then
                    state.isFiring = true
                    state.fireTicks = 0
                    state.fireDuration = 0 -- Reset fire duration when starting to fire
                    if not state.soundPlayed then
                        S_StartSound(player.mo, sfx_godzbm)
                        state.soundPlayed = true
                    end
                end

                state.fireTicks = state.fireTicks + 1
                state.fireDuration = state.fireDuration + 1

                if state.fireTicks >= FIRE_INTERVAL then
                    state.fireTicks = 0

                    local horizontal_offset = 50 * FRACUNIT
                    local vertical_offset = 70 * FRACUNIT
                    local spawnx = player.mo.x + FixedMul(cos(player.mo.angle), horizontal_offset)
                    local spawny = player.mo.y + FixedMul(sin(player.mo.angle), horizontal_offset)
                    local spawnz = player.mo.z + vertical_offset
                    local aim_angle = player.aiming

                    local heatBeam = P_SpawnMobj(spawnx, spawny, spawnz, MT_HEATBEAM)

                    if heatBeam and heatBeam.valid then
                        heatBeam.target = player.mo
                        heatBeam.angle = player.mo.angle
                        heatBeam.momx = FixedMul(cos(player.mo.angle), 50 * FRACUNIT)
                        heatBeam.momy = FixedMul(sin(player.mo.angle), 50 * FRACUNIT)
                        heatBeam.momz = FixedMul(sin(aim_angle), 50 * FRACUNIT)
                    end
                end

                if state.chargeTicks >= RING_DRAIN_DELAY then
                    state.ringDrainTicks = state.ringDrainTicks + 1
                    if state.ringDrainTicks >= RING_DRAIN_INTERVAL then
                        state.ringDrainTicks = 0
                        if player.rings > 1 then
                            player.rings = player.rings - 1
                        else
                            player.mo.sprite = SPR_PLAY
                            player.mo.frame = A
                            state.chargeTicks = 0
                            state.isFiring = false
                            state.fireTicks = 0
                            state.soundPlayed = false
                            state.fireDuration = 0
                            break
                        end
                    end
                end

                -- Stop firing if the fire duration exceeds MAX_FIRE_TIME (1 second)
                if state.fireDuration >= MAX_FIRE_TIME then
                    player.mo.sprite = SPR_PLAY
                    player.mo.frame = A
                    state.chargeTicks = 0
                    state.isFiring = false
                    state.fireTicks = 0
                    state.soundPlayed = false
                    state.fireDuration = 0
                end
            end
        else
            if state.chargeTicks > 0 then
                player.mo.state = S_PLAY_STND
                state.chargeTicks = 0
                state.isFiring = false
                state.fireTicks = 0
                state.soundPlayed = false
                state.ringDrainTicks = 0
                state.fireDuration = 0
            end
        end
    end
end

addHook("ThinkFrame", HeatBeam)