freeslot("S_GODZILLA_ROAR2", "SPR_SKR2")

states[S_GODZILLA_ROAR2] = {
    sprite = SPR_SKR2,
    frame = FF_ANIMATE,
    tics = 70,
    nextstate = S_PLAY_STND
}

-- Reset cooldown on respawn or map change
addHook("PlayerSpawn", function(player)
    if player.mo and player.mo.skin == "godzilla" then
        -- Reset roar cooldown when player respawns
        player.roarcooldown = 0
    end
end)

addHook("MapChange", function()
    for player in players.iterate() do
        if player.mo and player.mo.skin == "godzilla" then
            -- Reset roar cooldown when entering a new map
            player.roarcooldown = 0
        end
    end
end)

addHook("ThinkFrame", function()
    for player in players.iterate() do
        if player.mo and player.mo.skin == "godzilla" then
            -- Initialize timers if they don’t exist
            player.taunttimer = player.taunttimer or 0
            player.roarcooldown = player.roarcooldown or 0

            -- Decrease cooldown if it's active
            if player.roarcooldown > 0 then
                player.roarcooldown = player.roarcooldown - 1
            end

            -- Detect single press of Custom 2 button and check cooldown
            if (player.cmd.buttons & BT_CUSTOM2) and not (player.prevbuttons & BT_CUSTOM2) then
                if P_IsObjectOnGround(player.mo) 
                   and (player.mo.state == S_PLAY_STND or player.mo.state == S_PLAY_WAIT)
                   and player.roarcooldown == 0 then
                    -- Play roar sound and enter roaring state
                    S_StartSound(player.mo, sfx_godror)
                    player.mo.state = S_GODZILLA_ROAR2
                    player.taunttimer = 0  -- Reset 2-second animation timer
                    player.roarcooldown = 35 * 5  -- 5 seconds (35 tics per second)
                end
            end

            -- If roaring, stop all movement and jumping completely
            if player.mo.state == S_GODZILLA_ROAR2 then
                player.mo.momx = 0
                player.mo.momy = 0
                player.mo.momz = 0
                player.pflags = $ | PF_FULLSTASIS | PF_JUMPSTASIS -- Disable movement & jumping

                -- Count tics and return to standing state after 2 seconds (140 tics)
                player.taunttimer = $ + 1
                if player.taunttimer >= 140 then
                    player.mo.state = S_PLAY_STND
                    player.pflags = $ & ~(PF_FULLSTASIS | PF_JUMPSTASIS) -- Restore movement & jumping
                end
            end
        end

        -- Store previous button state to detect single press
        player.prevbuttons = player.cmd.buttons
    end
end)